@@ -7,10 +7,30 @@ import warnings
 import yaml
 from argparse import Action, Namespace
 from collections.abc import Iterable as abcIterable
+from collections.abc import Mapping as abcMapping
+from collections.abc import MutableMapping as abcMutableMapping
+from collections.abc import Set as abcSet
+from collections.abc import MutableSet as abcMutableSet
 from collections.abc import Sequence as abcSequence
+from collections.abc import MutableSequence as abcMutableSequence
 from enum import Enum
 from functools import partial
-from typing import Any, Dict, Iterable, List, Sequence, Set, Tuple, Type, TypeVar, Union
+from typing import (
+    Any,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    MutableMapping,
+    MutableSequence,
+    MutableSet,
+    Sequence,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
 try:
     from typing import Literal  # type: ignore
@@ -50,10 +70,10 @@ root_types = {
     Literal,
     Type, type,
     Union,
-    List, list, Iterable, Sequence, abcIterable, abcSequence,
+    List, list, Iterable, Sequence, MutableSequence, abcIterable, abcSequence, abcMutableSequence,
     Tuple, tuple,
-    Set, set,
-    Dict, dict,
+    Set, set, frozenset, MutableSet, abcMutableSet,
+    Dict, dict, Mapping, MutableMapping, abcMapping, abcMutableMapping,
 }
 
 leaf_types = {
@@ -184,6 +204,14 @@ class ActionTypeHint(Action):
             cfg, val, opt_str = args[1:]
             if isinstance(opt_str, str):
                 if opt_str.endswith('.class_path'):
+                    cfg_dest = get_key_value_from_flat_dict(vars(cfg), self.dest)
+                    if cfg_dest.get('class_path') == val:
+                        return
+                    elif cfg_dest.get('init_args') is not None and cfg_dest.get('init_args') != {}:
+                        warnings.warn(
+                            'Argument ' + opt_str + '=' + val + ' implies discarding init_args ' + str(cfg_dest.get('init_args')) +
+                            ' defined for class_path ' + cfg_dest.get('class_path')
+                        )
                     val = {'class_path': val}
                 elif '.init_args.' in opt_str:
                     match = re.match(r'.+\.init_args\.([^.]+)$', opt_str)
@@ -192,8 +220,7 @@ class ActionTypeHint(Action):
                         init_args[match.groups()[0]] = val
                         val = getattr(cfg, self.dest)
                     else:
-                        cfg_dict = vars(cfg)
-                        cfg_dest = get_key_value_from_flat_dict(cfg_dict, self.dest)
+                        cfg_dest = get_key_value_from_flat_dict(vars(cfg), self.dest)
                         init_args = cfg_dest.get('init_args', {})
                         init_args[match.groups()[0]] = val
                         val = cfg_dest
@@ -326,11 +353,11 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, s
     elif typehint in {Type, type} or typehint_origin in {Type, type}:
         if serialize:
             val = object_path_serializer(val)
-        elif not serialize and not isinstance(val, Type):
+        elif not serialize and not isinstance(val, type):
             path = val
             val = import_object(val)
-            if (typehint in {Type, type} and not isinstance(val, (Type, type))) or \
-               (typehint != Type and typehint_origin in {Type, type} and not _issubclass(val, subtypehints[0])):
+            if (typehint in {Type, type} and not isinstance(val, type)) or \
+               (typehint not in {Type, type} and not _issubclass(val, subtypehints[0])):
                 raise ValueError('Value "'+str(path)+'" is not a '+str(typehint)+'.')
 
     # Union
@@ -348,7 +375,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, s
         val = [v for v in vals if not isinstance(v, Exception)][0]
 
     # Tuple or Set
-    elif typehint_origin in {Tuple, tuple, Set, set}:
+    elif typehint_origin in {Tuple, tuple, Set, set, frozenset, MutableSet, abcSet, abcMutableSet}:
         if not isinstance(val, (list, tuple, set)):
             raise ValueError('Expected a '+str(typehint_origin)+' but got "'+str(val)+'"')
         val = list(val)
@@ -366,7 +393,8 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, s
             val = tuple(val) if typehint_origin in {Tuple, tuple} else set(val)
 
     # List, Iterable or Sequence
-    elif typehint_origin in {List, list, Iterable, Sequence, abcIterable, abcSequence}:
+    elif typehint_origin in {List, list, Iterable, Sequence, MutableSequence, abcIterable, abcSequence,
+                             abcMutableSequence}:
         if not isinstance(val, list):
             raise ValueError('Expected a List but got "'+str(val)+'"')
         if subtypehints is not None:
@@ -374,7 +402,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, s
                 val[n] = adapt_typehints(v, subtypehints[0], **adapt_kwargs)
 
     # Dict
-    elif typehint_origin in {Dict, dict}:
+    elif typehint_origin in {Dict, dict, Mapping, MutableMapping, abcMapping, abcMutableMapping}:
         if not isinstance(val, dict):
             raise ValueError('Expected a Dict but got "'+str(val)+'"')
         if subtypehints is not None:
@@ -402,6 +430,8 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, s
                     'string ' + val + '. If this was set as a default, consider setting a dict or using lazy_instance.'
                 )
             return val
+        if serialize and isinstance(val, str):
+            return val
         if not (isinstance(val, str) or (isinstance(val, dict) and 'class_path' in val) or (isinstance(val, Namespace) and hasattr(val, 'class_path'))):
             raise ValueError('Type '+str(typehint)+' expects an str or a Dict with a class_path entry but got "'+str(val)+'"')
         try:
